/*
 * Created on 2 mai 2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package org.injac.oai.server.crosswalk;

import org.injac.oai.format.*;

import java.util.*;
import org.apache.log4j.Logger;
import org.oclc.oai.server.crosswalk.Crosswalk;
import org.oclc.oai.server.verb.CannotDisseminateFormatException;

/**
 * @author Franois Jannin
 *
 * Generic Crosswalk for generating metadatas from a format XML file
 */
public class WebdavCrosswalk extends Crosswalk{
	/**
	 * Static logger
	 */
	static Logger logger = Logger.getLogger(WebdavCrosswalk.class);
	private String formatFile = null;
	private FormatConfig formatConfig = null;
	

    /**
     * The constructor assigns the schemaLocation associated with this crosswalk. Since
     * the crosswalk is trivial in this case, no properties are utilized.
     *
     * @param properties properties that are needed to configure the crosswalk.
     */
    public WebdavCrosswalk(String formatfile, String schemaLoc) {
	//"http://www.openarchives.org/OAI/2.0/oai_dc/ http://www.openarchives.org/OAI/2.0/oai_dc.xsd"
    	super(schemaLoc);
    	formatFile = formatfile;
    	initContext(formatfile);
	
    }

    /**
     * Can this nativeItem be represented in this format?
     * @param nativeItem a record in native format
     * @return true if DC format is possible, false otherwise.
     */
    public  boolean isAvailableFor(Object nativeItem) {
    	
	if (nativeItem != null)
	    return true;
	return false;
    }

	 /**
     * Perform the actual crosswalk.
     *
     * @param nativeItem the native "item". In this case, it is
     * already formatted as an OAI <record> element, with the
     * possible exception that multiple metadataFormats are
     * present in the <metadata> element.
     * @return a String containing the XML to be stored within the <metadata> element.
     * @exception CannotDisseminateFormatException nativeItem doesn't support this format.
     */
    public  String createMetadata(Object nativeItem)
	throws CannotDisseminateFormatException {
    	try {
    			// build metadata from xml map file
    			return parseFormat(nativeItem);
    	}
    	catch(Exception e)
		{
    		//logger.error("Webdav2oai_dc::createMetadata : "+e.getMessage());
    		throw new CannotDisseminateFormatException(formatConfig.getMetadataPrefix());
		}
    	
    	
    }
	private void initContext(String formatfile)
	{
		try{
			formatConfig = new FormatConfig(formatFile);
		}catch(Exception e){
			logger.error(e+ " : " +e.getLocalizedMessage());
			return;
		}
        logger.debug("create format config object with root :"+ formatConfig.getRootTag());
	
	}
	
	/** 
	 * Parse map file and build metadatas from nativeItem
	 * @param nativeItem
	 * @return metadata element of a record as string
	 */
	private String parseFormat(Object nativeItem)throws CannotDisseminateFormatException
	{
		logger.debug("entering parseFormat");
		
    	String result = "";
    	
    	HashMap hash = (HashMap)nativeItem;
        //
        try
        {
        	ArrayList mapList = formatConfig.getMappingList();
        	ArrayList tagList = new ArrayList();
        	for(int nodeIndex = 0; nodeIndex < mapList.size();nodeIndex++)
            {
              try
              {
              	 MappingAttribute mappAttr = (MappingAttribute) mapList.get(nodeIndex);
              	 ArrayList maptagList =  parseMap(mappAttr, hash); 
           		 tagList.addAll(maptagList);
              }catch(CannotDisseminateFormatException ce){
              	throw ce;
              }
              catch(Exception exp)
              {
              	logger.error(exp.toString());
                throw new CannotDisseminateFormatException(formatConfig.getMetadataPrefix());
              }
              
              
           }
        	result=mergeTagList(tagList);
        }catch(CannotDisseminateFormatException ce){
        	throw ce;
        }
        catch(Exception exp)
        {
        	logger.error(exp.getClass().getName()+ " : "+ exp.getMessage());
            throw new CannotDisseminateFormatException(formatConfig.getMetadataPrefix());
        	
        }
       return result;
       
	}
	
	
	/**
	 * Create tags according to MappingAttribute simple object properties
	 * @param mapAttr
	 * @param hash
	 * @return an array List of tags 
	 * @throws CannotDisseminateFormatException
	 */
	private ArrayList parseMap(MappingAttribute mapAttr, HashMap hash)throws CannotDisseminateFormatException{  
	
	ArrayList result = new ArrayList();
	
	String sourceName = mapAttr.getSource();
	 
    // get webdav value from hashmap
    String S = null;
    if(sourceName!=null)
    	S = (String)hash.get(sourceName);
    boolean canDisseminate = true;
    	
    // test disseminating possibility with required args
	 if((mapAttr.getRequired()!=null) && (mapAttr.getRequired().equals("yes")))
	 {
	 	canDisseminate= canDisseminate && ((S != null) && (S.length() > 0)); 
	 }
	 if(!canDisseminate)
		 throw new CannotDisseminateFormatException(formatConfig.getMetadataPrefix());
	 // special date conversion	
    if(mapAttr.getDest().equals("etdms:date"))
    	S = UTCtoETDMSDate(S);
    
    logger.debug("webdavName :" + sourceName +"\t desFormatName : "+mapAttr.getDest());
    
    // noeud avec valeur non nulle
   // if(S != null && S.length() > 0) 
   // {
    	if((mapAttr.getMultiParse() ==null) ||(mapAttr.getMultiParse().equals("")))
    	{	
    		DestTag tag = parseDestFormatName(mapAttr, S);
    		// rcursivit pour les mapping imbriqus
    		if((mapAttr.getGroup()!=null) && ((mapAttr.getGroup().equals("true")))){
				 tag.children = new ArrayList();
					for(int i = 0; i< mapAttr.getChildrenCount();i++){
						tag.children.addAll(parseMap(mapAttr.getChild(i), hash));
					}
			 }
    		result.add(tag);
    	}
    	else if(S != null && S.length() > 0) {
    		StringTokenizer st = new StringTokenizer(S, mapAttr.getMultiParse());
    		while(st.hasMoreTokens())
    		{
    			DestTag tag= parseDestFormatName(mapAttr, st.nextToken());
    			result.add(tag);
    		}
    	}
    		
    //}
    return result;
}
	
	
	
	private String UTCtoETDMSDate(String utc){
		if(utc ==null)
			return null;
		String res = utc.substring(0,10);
		return res;
	}
	/**
	 * Parse a XPATH and convert it into XML tags structure with value on the inner element
	 * @param dest
	 * @param value
	 * @return xml fragment
	 */
	private DestTag parseDestFormatName(MappingAttribute mapAttr, String value) {
		String attributes = mapAttr.getAttributes();
		String defaults = mapAttr.getDefaults();
		String dest = mapAttr.getDest();
		if(logger.isDebugEnabled()){
			logger.debug("entering parseDestFormat : dest :" + dest);
		}
		HashMap attrMap =null;
		if(!attributes.equals(""))
        {
         	//parseAttributes(S);
		 	attrMap = new HashMap();
		 	value= parseAttributes(attributes, value, attrMap, defaults);
        }
		int last=dest.lastIndexOf("/");
		DestTag tag = new DestTag();
		// prefix defaults to global prefix
		String prefix= formatConfig.getMetadataPrefix();
		int pref_index= dest.indexOf(":");
		if(pref_index!=-1)
			prefix=dest.substring(0,pref_index);
		// no path tag
		if(last==-1)
		{
			logger.debug("no path tag");
			tag.prefix = prefix;
			tag.tag = dest.substring(pref_index+1);
			tag.XBase="";
			tag.value=value;
			tag.attrMap = attrMap;
			
		}
		// tag with path
		else
		{	
			logger.debug("tag with path");
			tag.prefix = prefix;
			tag.tag = dest.substring(last+1);
			tag.XBase=dest.substring(pref_index+1, last);
			tag.value=value;
			tag.attrMap = attrMap;
			StringTokenizer st = new StringTokenizer(tag.XBase,"/");
		// 	build list of elements
			ArrayList elements= new ArrayList();
			while(st.hasMoreElements())
			{
				String path_element = (String)st.nextToken();
				elements.add(path_element);
				if(logger.isDebugEnabled())
					logger.debug("parseDestFormat : path element :" + path_element);
			}
			tag.XBaseArray = elements;
		}
		String debug = "parseDestFormat for " +dest;
		debug+=tag;
		if(logger.isDebugEnabled())
			logger.debug("parseDestFormat for:"+dest+tag);
		
		 
		return tag;
	}
	/**
	 * Parse attributes
	 * @param attributes string with attributes names to parse 
	 * @param value String to parse 
	 * @param en empty HashMap 
	 * @param defaults : default values to parse
	 * @return new value without accodances
	 * 
	 */
	private String parseAttributes(String attributes, String value, HashMap attrMap,String defaults){
		
		int accIndex=value.indexOf("{");
		int endIndex=value.indexOf("}");
	 	
		//		 parse names
 		ArrayList attrNames = new ArrayList();
 		StringTokenizer stNames = new StringTokenizer(attributes,",");
 		while(stNames.hasMoreTokens()){
 			attrNames.add(stNames.nextToken());
 		}
 		// parse default values
 		ArrayList defaultValues  = new ArrayList();
 		StringTokenizer stDefaults = new StringTokenizer(defaults,",");
 		while(stDefaults.hasMoreTokens()){
 			defaultValues.add(stDefaults.nextToken());
 		}
 		ArrayList attrValues = null;
		
		// detect accodances for values separated by comma to feed attributes with
		if((accIndex !=-1) && (endIndex!= -1))
	 	{
	 		// parse values
	 		attrValues = new ArrayList();
	 		String values = value.substring(accIndex+1, endIndex);
	 		StringTokenizer stValues = new StringTokenizer(values,",");
	 		while(stValues.hasMoreTokens()){
	 			attrValues.add(stValues.nextToken());
	 		}
	 		// remove accodances from value
	 		value=value.substring(endIndex+1);
	 		
	 	}
 		// fill attributes hashmap
 			for(int i=0;i<attrNames.size();i++)
	 		{
 				if((attrValues != null)&& (i< attrValues.size())){
 					// 1. optimal case : feed with values provided	
 					attrMap.put(attrNames.get(i), attrValues.get(i));
 				}else if(i< defaultValues.size()){
 					// 2. second case : feed with default values
 					attrMap.put(attrNames.get(i), defaultValues.get(i));
 				}else{
 					// 3. degraded case : attribute not inserted 
 					// NOP
 				}
	 		}
 		
	 	return value;
	}
	
	
	/**
	 * Generate XML structure from Destination tags
	 * @param tagList
	 * @return
	 */
	private String mergeTagList(ArrayList tagList){
		logger.debug("entering mergeTagList array size = "+tagList.size());
		
		
		MetadataDocument ametaDoc = new MetadataDocument(formatConfig.getRootTag(), formatConfig.getMetadataPrefix(), formatConfig.getMetadataNamespace(), formatConfig.getSchema(), formatConfig.getNamespaces());
		ametaDoc.mergeTagList(tagList);
		
		return ametaDoc.getDocumentAsString();
		
	}
	
	
	/**
	 * inner class for describing a Destination tag
	 * @author F.Jannin
	 *
	 * TODO make constructor, accessors with trigger for countpath recomputing  
	 */
	
	class DestTag implements Comparable {
		public String prefix;
		public String XBase;
		public String tag;
		public String value;
		public ArrayList XBaseArray=null;
		public HashMap attrMap=null;
		public ArrayList children = null;
		/**
		 * return 0 if Xbase path are the same
		 * return -1 if path is nearer from root element
		 * return +1 if path is farer from root element 
		 */
		public int compareTo(Object o) {
			if(countPath() < ((DestTag)o).countPath())
				return -1;
			if(countPath() > ((DestTag)o).countPath())
				return 1;
			return 0;
		}
		private int countPath(){
			StringTokenizer st = new StringTokenizer(XBase,"/");
			int count=0;
			while(st.hasMoreElements())
				count++;
			return count;
		}
		public String toString(){
			String result="";
			result+="\nprefix :" + this.prefix;
			result+="\ntag :" + this.tag;
			result+="\nXBase :" + this.XBase;
			result+="\nvalue :" + this.value;
			if(this.XBaseArray != null)
				result+="\narray size :" + this.XBaseArray.size();
			else
				result+="\nNULL array";
			return result;
		}
	}
}
